package org.basex.query.expr;

import org.basex.query.*;
import org.basex.query.item.*;
import org.basex.query.iter.*;

/**
 * Iterative filter expression without numeric predicates.
 *
 * @author BaseX Team 2005-12, BSD License
 * @author Christian Gruen
 */
final class IterFilter extends Filter {
  /**
   * Constructor.
   * @param f original filter
   */
  IterFilter(final Filter f) {
    super(f.info, f.root, f.preds);
    type = f.type;
  }

  @Override
  public Iter iter(final QueryContext ctx) {
    return new Iter() {
      /** Iterator. */
      Iter iter;

      @Override
      public Item next() throws QueryException {
        // first call - initialize iterator
        if(iter == null) iter = ctx.iter(root);

        // cache context
        final Value cv = ctx.value;
        final long cp = ctx.pos;
        final long cs = ctx.size;
        try {
          for(Item it; (it = iter.next()) != null;) {
            ctx.checkStop();
            if(preds(it, ctx)) return it;
          }
          return null;
        } finally {
          ctx.value = cv;
          ctx.pos = cp;
          ctx.size = cs;
        }
      }
    };
  }
}
